@@ -6,9 +6,9 @@ if 0;
 
 # ============================================================================
 
-# $Id: snmpwalk.pl,v 2.2 2003/05/06 11:00:46 dtown Exp $
+# $Id: snmpwalk.pl,v 2.3 2004/07/20 13:38:01 dtown Exp $
 
-# Copyright (c) 2000-2003 David M. Town <dtown@cpan.org>
+# Copyright (c) 2000-2004 David M. Town <dtown@cpan.org>
 # All rights reserved.
 
 # This program is free software; you may redistribute it and/or modify it
@@ -16,17 +16,17 @@ if 0;
 
 # ============================================================================
 
-use Net::SNMP 4.1.0 qw(oid_lex_sort oid_base_match SNMP_VERSION_1 DEBUG_ALL);
+use Net::SNMP v5.0.0 qw(:snmp DEBUG_ALL);
 use Getopt::Std;
 
 use strict;
 use vars qw($SCRIPT $VERSION %OPTS);
 
 $SCRIPT  = 'snmpwalk';
-$VERSION = '2.1.0';
+$VERSION = '2.2.0';
 
 # Validate the command line options
-if (!getopts('a:A:c:dE:m:n:p:r:t:u:v:x:X:', \%OPTS)) {
+if (!getopts('a:A:c:dD:E:m:n:p:r:t:u:v:x:X:', \%OPTS)) {
    _usage();
 }
 
@@ -41,6 +41,7 @@ my ($s, $e) = Net::SNMP->session(
    exists($OPTS{a}) ? (-authprotocol =>  $OPTS{a}) : (),
    exists($OPTS{A}) ? (-authpassword =>  $OPTS{A}) : (),
    exists($OPTS{c}) ? (-community    =>  $OPTS{c}) : (),
+   exists($OPTS{D}) ? (-domain       =>  $OPTS{D}) : (),
    exists($OPTS{d}) ? (-debug        => DEBUG_ALL) : (),
    exists($OPTS{m}) ? (-maxmsgsize   =>  $OPTS{m}) : (),
    exists($OPTS{p}) ? (-port         =>  $OPTS{p}) : (),
@@ -72,10 +73,14 @@ if ($s->version == SNMP_VERSION_1) {
    my $oid;
 
    while (defined($s->get_next_request(@args))) {
-      $oid = ($s->var_bind_names)[0];
+      $oid = ($s->var_bind_names())[0];
 
       if (!oid_base_match($ARGV[0], $oid)) { last; }
-      printf("%s => %s\n", $oid, $s->var_bind_list->{$oid});   
+      printf(
+          "%s = %s: %s\n", $oid, 
+          snmp_type_ntop($s->var_bind_types()->{$oid}), 
+          $s->var_bind_list()->{$oid}, 
+      );
 
       @args = (-varbindlist => [$oid]);
    }
@@ -86,15 +91,19 @@ if ($s->version == SNMP_VERSION_1) {
 
    outer: while (defined($s->get_bulk_request(@args))) {
 
-      my @oids = oid_lex_sort(keys(%{$s->var_bind_list}));
+      my @oids = oid_lex_sort(keys(%{$s->var_bind_list()}));
 
       foreach (@oids) {
 
          if (!oid_base_match($ARGV[0], $_)) { last outer; }
-         printf("%s => %s\n", $_, $s->var_bind_list->{$_});
+         printf(
+            "%s = %s: %s\n", $_, 
+            snmp_type_ntop($s->var_bind_types()->{$_}),
+            $s->var_bind_list()->{$_},
+         );
 
          # Make sure we have not hit the end of the MIB
-         if ($s->var_bind_list->{$_} eq 'endOfMibView') { last outer; } 
+         if ($s->var_bind_list()->{$_} eq 'endOfMibView') { last outer; } 
       }
 
       # Get the last OBJECT IDENTIFIER in the returned list
@@ -139,6 +148,7 @@ Options: -v 1|2c|3      SNMP version
          -x <privproto> Privacy protocol <des|3des|aes128|aes192|aes256>
          -X <password>  Privacy password
    Transport Layer:
+         -D <domain>    Domain <udp4|udp6|tcp4|tcp6>
          -m <octets>    Maximum message size
          -p <port>      Destination UDP port
          -r <attempts>  Number of retries
